# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: session.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rsession.proto\x12\rnidevice_grpc\"2\n\x07Session\x12\x0e\n\x04name\x18\x01 \x01(\tH\x00\x12\x0c\n\x02id\x18\x02 \x01(\rH\x00\x42\t\n\x07session\"j\n\x10\x44\x65viceProperties\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05model\x18\x02 \x01(\t\x12\x0e\n\x06vendor\x18\x03 \x01(\t\x12\x15\n\rserial_number\x18\x04 \x01(\t\x12\x12\n\nproduct_id\x18\x05 \x01(\r\"\x19\n\x17\x45numerateDevicesRequest\"L\n\x18\x45numerateDevicesResponse\x12\x30\n\x07\x64\x65vices\x18\x01 \x03(\x0b\x32\x1f.nidevice_grpc.DeviceProperties\";\n\x0eReserveRequest\x12\x16\n\x0ereservation_id\x18\x01 \x01(\t\x12\x11\n\tclient_id\x18\x02 \x01(\t\"&\n\x0fReserveResponse\x12\x13\n\x0bis_reserved\x18\x01 \x01(\x08\"F\n\x19IsReservedByClientRequest\x12\x16\n\x0ereservation_id\x18\x01 \x01(\t\x12\x11\n\tclient_id\x18\x02 \x01(\t\"1\n\x1aIsReservedByClientResponse\x12\x13\n\x0bis_reserved\x18\x01 \x01(\x08\"=\n\x10UnreserveRequest\x12\x16\n\x0ereservation_id\x18\x01 \x01(\t\x12\x11\n\tclient_id\x18\x02 \x01(\t\"*\n\x11UnreserveResponse\x12\x15\n\ris_unreserved\x18\x01 \x01(\x08\"\x14\n\x12ResetServerRequest\".\n\x13ResetServerResponse\x12\x17\n\x0fis_server_reset\x18\x01 \x01(\x08*\xbc\x01\n\x1dSessionInitializationBehavior\x12/\n+SESSION_INITIALIZATION_BEHAVIOR_UNSPECIFIED\x10\x00\x12\x32\n.SESSION_INITIALIZATION_BEHAVIOR_INITIALIZE_NEW\x10\x01\x12\x36\n2SESSION_INITIALIZATION_BEHAVIOR_ATTACH_TO_EXISTING\x10\x02\x32\xd2\x03\n\x10SessionUtilities\x12\x63\n\x10\x45numerateDevices\x12&.nidevice_grpc.EnumerateDevicesRequest\x1a\'.nidevice_grpc.EnumerateDevicesResponse\x12H\n\x07Reserve\x12\x1d.nidevice_grpc.ReserveRequest\x1a\x1e.nidevice_grpc.ReserveResponse\x12i\n\x12IsReservedByClient\x12(.nidevice_grpc.IsReservedByClientRequest\x1a).nidevice_grpc.IsReservedByClientResponse\x12N\n\tUnreserve\x12\x1f.nidevice_grpc.UnreserveRequest\x1a .nidevice_grpc.UnreserveResponse\x12T\n\x0bResetServer\x12!.nidevice_grpc.ResetServerRequest\x1a\".nidevice_grpc.ResetServerResponseBB\n\x12\x63om.ni.grpc.deviceB\x08NiDeviceP\x01\xaa\x02\x1fNationalInstruments.Grpc.Deviceb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'session_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\022com.ni.grpc.deviceB\010NiDeviceP\001\252\002\037NationalInstruments.Grpc.Device'
  _SESSIONINITIALIZATIONBEHAVIOR._serialized_start=699
  _SESSIONINITIALIZATIONBEHAVIOR._serialized_end=887
  _SESSION._serialized_start=32
  _SESSION._serialized_end=82
  _DEVICEPROPERTIES._serialized_start=84
  _DEVICEPROPERTIES._serialized_end=190
  _ENUMERATEDEVICESREQUEST._serialized_start=192
  _ENUMERATEDEVICESREQUEST._serialized_end=217
  _ENUMERATEDEVICESRESPONSE._serialized_start=219
  _ENUMERATEDEVICESRESPONSE._serialized_end=295
  _RESERVEREQUEST._serialized_start=297
  _RESERVEREQUEST._serialized_end=356
  _RESERVERESPONSE._serialized_start=358
  _RESERVERESPONSE._serialized_end=396
  _ISRESERVEDBYCLIENTREQUEST._serialized_start=398
  _ISRESERVEDBYCLIENTREQUEST._serialized_end=468
  _ISRESERVEDBYCLIENTRESPONSE._serialized_start=470
  _ISRESERVEDBYCLIENTRESPONSE._serialized_end=519
  _UNRESERVEREQUEST._serialized_start=521
  _UNRESERVEREQUEST._serialized_end=582
  _UNRESERVERESPONSE._serialized_start=584
  _UNRESERVERESPONSE._serialized_end=626
  _RESETSERVERREQUEST._serialized_start=628
  _RESETSERVERREQUEST._serialized_end=648
  _RESETSERVERRESPONSE._serialized_start=650
  _RESETSERVERRESPONSE._serialized_end=696
  _SESSIONUTILITIES._serialized_start=890
  _SESSIONUTILITIES._serialized_end=1356
# @@protoc_insertion_point(module_scope)



# Do not edit this file; it was automatically generated.

from nidaqmx.task.channels._channel import Channel
from nidaqmx.constants import (
    ConstrainedGenMode, DataTransferActiveTransferMode, Edge, FrequencyUnits,
    Level, OutputDataTransferCondition, TimeUnits, UsageTypeCO)


class COChannel(Channel):
    """
    Represents one or more counter output virtual channels and their properties.
    """
    __slots__ = ()

    def __repr__(self):
        return f'COChannel(name={self._name})'

    @property
    def co_auto_incr_cnt(self):
        """
        int: Specifies a number of timebase ticks by which to increase
            the time spent in the idle state for each successive pulse.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x295)
        return val

    @co_auto_incr_cnt.setter
    def co_auto_incr_cnt(self, val):
        self._interpreter.set_chan_attribute_uint32(self._handle, self._name, 0x295, val)

    @co_auto_incr_cnt.deleter
    def co_auto_incr_cnt(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x295)

    @property
    def co_constrained_gen_mode(self):
        """
        :class:`nidaqmx.constants.ConstrainedGenMode`: Specifies
            constraints to apply when the counter generates pulses.
            Constraining the counter reduces the device resources
            required for counter operation. Constraining the counter can
            also allow additional analog or counter tasks on the device
            to run concurrently. For continuous counter tasks, NI-DAQmx
            consumes no device resources when the counter is
            constrained. For finite counter tasks, resource use
            increases with the frequency regardless of the constraint
            mode. However, fixed frequency constraints significantly
            reduce resource usage, and fixed duty cycle constraint
            marginally reduces it.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x29f2)
        return ConstrainedGenMode(val)

    @co_constrained_gen_mode.setter
    def co_constrained_gen_mode(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x29f2, val)

    @co_constrained_gen_mode.deleter
    def co_constrained_gen_mode(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x29f2)

    @property
    def co_count(self):
        """
        int: Indicates the current value of the count register.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x293)
        return val

    @property
    def co_ctr_timebase_active_edge(self):
        """
        :class:`nidaqmx.constants.Edge`: Specifies whether a timebase
            cycle is from rising edge to rising edge or from falling
            edge to falling edge.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x341)
        return Edge(val)

    @co_ctr_timebase_active_edge.setter
    def co_ctr_timebase_active_edge(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x341, val)

    @co_ctr_timebase_active_edge.deleter
    def co_ctr_timebase_active_edge(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x341)

    @property
    def co_ctr_timebase_dig_fltr_enable(self):
        """
        bool: Specifies whether to apply the pulse width filter to the
            signal.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x2276)
        return val

    @co_ctr_timebase_dig_fltr_enable.setter
    def co_ctr_timebase_dig_fltr_enable(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x2276, val)

    @co_ctr_timebase_dig_fltr_enable.deleter
    def co_ctr_timebase_dig_fltr_enable(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2276)

    @property
    def co_ctr_timebase_dig_fltr_min_pulse_width(self):
        """
        float: Specifies in seconds the minimum pulse width the filter
            recognizes.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x2277)
        return val

    @co_ctr_timebase_dig_fltr_min_pulse_width.setter
    def co_ctr_timebase_dig_fltr_min_pulse_width(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x2277, val)

    @co_ctr_timebase_dig_fltr_min_pulse_width.deleter
    def co_ctr_timebase_dig_fltr_min_pulse_width(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2277)

    @property
    def co_ctr_timebase_dig_fltr_timebase_rate(self):
        """
        float: Specifies in hertz the rate of the pulse width filter
            timebase. NI-DAQmx uses this value to compute settings for
            the filter.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x2279)
        return val

    @co_ctr_timebase_dig_fltr_timebase_rate.setter
    def co_ctr_timebase_dig_fltr_timebase_rate(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x2279, val)

    @co_ctr_timebase_dig_fltr_timebase_rate.deleter
    def co_ctr_timebase_dig_fltr_timebase_rate(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2279)

    @property
    def co_ctr_timebase_dig_fltr_timebase_src(self):
        """
        str: Specifies the input terminal of the signal to use as the
            timebase of the pulse width filter.
        """

        val = self._interpreter.get_chan_attribute_string(self._handle, self._name, 0x2278)
        return val

    @co_ctr_timebase_dig_fltr_timebase_src.setter
    def co_ctr_timebase_dig_fltr_timebase_src(self, val):
        self._interpreter.set_chan_attribute_string(self._handle, self._name, 0x2278, val)

    @co_ctr_timebase_dig_fltr_timebase_src.deleter
    def co_ctr_timebase_dig_fltr_timebase_src(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2278)

    @property
    def co_ctr_timebase_dig_sync_enable(self):
        """
        bool: Specifies whether to synchronize recognition of
            transitions in the signal to the internal timebase of the
            device.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x227a)
        return val

    @co_ctr_timebase_dig_sync_enable.setter
    def co_ctr_timebase_dig_sync_enable(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x227a, val)

    @co_ctr_timebase_dig_sync_enable.deleter
    def co_ctr_timebase_dig_sync_enable(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x227a)

    @property
    def co_ctr_timebase_master_timebase_div(self):
        """
        int: Specifies the divisor for an external counter timebase. You
            can divide the counter timebase in order to generate slower
            signals without causing the count register to roll over.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x18c3)
        return val

    @co_ctr_timebase_master_timebase_div.setter
    def co_ctr_timebase_master_timebase_div(self, val):
        self._interpreter.set_chan_attribute_uint32(self._handle, self._name, 0x18c3, val)

    @co_ctr_timebase_master_timebase_div.deleter
    def co_ctr_timebase_master_timebase_div(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x18c3)

    @property
    def co_ctr_timebase_rate(self):
        """
        float: Specifies in Hertz the frequency of the counter timebase.
            Specifying the rate of a counter timebase allows you to
            define output pulses in seconds rather than in ticks of the
            timebase. If you use an external timebase and do not specify
            the rate, you can define output pulses only in ticks of the
            timebase.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x18c2)
        return val

    @co_ctr_timebase_rate.setter
    def co_ctr_timebase_rate(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x18c2, val)

    @co_ctr_timebase_rate.deleter
    def co_ctr_timebase_rate(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x18c2)

    @property
    def co_ctr_timebase_src(self):
        """
        str: Specifies the terminal of the timebase to use for the
            counter. Typically, NI-DAQmx uses one of the internal
            counter timebases when generating pulses. Use this property
            to specify an external timebase and produce custom pulse
            widths that are not possible using the internal timebases.
        """

        val = self._interpreter.get_chan_attribute_string(self._handle, self._name, 0x339)
        return val

    @co_ctr_timebase_src.setter
    def co_ctr_timebase_src(self, val):
        self._interpreter.set_chan_attribute_string(self._handle, self._name, 0x339, val)

    @co_ctr_timebase_src.deleter
    def co_ctr_timebase_src(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x339)

    @property
    def co_data_xfer_mech(self):
        """
        :class:`nidaqmx.constants.DataTransferActiveTransferMode`:
            Specifies the data transfer mode for the device. For
            buffered operations, use DMA or USB Bulk. For non-buffered
            operations, use Polled.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x2ecc)
        return DataTransferActiveTransferMode(val)

    @co_data_xfer_mech.setter
    def co_data_xfer_mech(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x2ecc, val)

    @co_data_xfer_mech.deleter
    def co_data_xfer_mech(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2ecc)

    @property
    def co_data_xfer_req_cond(self):
        """
        :class:`nidaqmx.constants.OutputDataTransferCondition`:
            Specifies under what condition to transfer data from the
            buffer to the onboard memory of the device.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x2ecd)
        return OutputDataTransferCondition(val)

    @co_data_xfer_req_cond.setter
    def co_data_xfer_req_cond(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x2ecd, val)

    @co_data_xfer_req_cond.deleter
    def co_data_xfer_req_cond(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2ecd)

    @property
    def co_enable_initial_delay_on_retrigger(self):
        """
        bool: Specifies whether to apply the initial delay to
            retriggered pulse trains.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x2ec9)
        return val

    @co_enable_initial_delay_on_retrigger.setter
    def co_enable_initial_delay_on_retrigger(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x2ec9, val)

    @co_enable_initial_delay_on_retrigger.deleter
    def co_enable_initial_delay_on_retrigger(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2ec9)

    @property
    def co_mem_map_enable(self):
        """
        bool: Specifies for NI-DAQmx to map hardware registers to the
            memory space of the application, if possible. Normally, NI-
            DAQmx maps hardware registers to memory accessible only to
            the kernel. Mapping the registers to the memory space of the
            application increases performance. However, if the
            application accesses the memory space mapped to the
            registers, it can adversely affect the operation of the
            device and possibly result in a system crash.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x2ed3)
        return val

    @co_mem_map_enable.setter
    def co_mem_map_enable(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x2ed3, val)

    @co_mem_map_enable.deleter
    def co_mem_map_enable(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2ed3)

    @property
    def co_output_state(self):
        """
        :class:`nidaqmx.constants.Level`: Indicates the current state of
            the output terminal of the counter.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x294)
        return Level(val)

    @property
    def co_output_type(self):
        """
        :class:`nidaqmx.constants.UsageTypeCO`: Indicates how to define
            pulses generated on the channel.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x18b5)
        return UsageTypeCO(val)

    @property
    def co_prescaler(self):
        """
        int: Specifies the divisor to apply to the signal you connect to
            the counter source terminal. Pulse generations defined by
            frequency or time take this setting into account, but pulse
            generations defined by ticks do not. You should use a
            prescaler only when you connect an external signal to the
            counter source terminal and when that signal has a higher
            frequency than the fastest onboard timebase.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x226d)
        return val

    @co_prescaler.setter
    def co_prescaler(self, val):
        self._interpreter.set_chan_attribute_uint32(self._handle, self._name, 0x226d, val)

    @co_prescaler.deleter
    def co_prescaler(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x226d)

    @property
    def co_pulse_done(self):
        """
        bool: Indicates if the task completed pulse generation. Use this
            value for retriggerable pulse generation when you need to
            determine if the device generated the current pulse. For
            retriggerable tasks, when you query this property, NI-DAQmx
            resets it to False.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x190e)
        return val

    @property
    def co_pulse_duty_cyc(self):
        """
        float: Specifies the duty cycle of the pulses. The duty cycle of
            a signal is the width of the pulse divided by period. NI-
            DAQmx uses this ratio and the pulse frequency to determine
            the width of the pulses and the delay between pulses.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x1176)
        return val

    @co_pulse_duty_cyc.setter
    def co_pulse_duty_cyc(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x1176, val)

    @co_pulse_duty_cyc.deleter
    def co_pulse_duty_cyc(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1176)

    @property
    def co_pulse_freq(self):
        """
        float: Specifies the frequency of the pulses to generate. This
            value is in the units you specify with
            **co_pulse_freq_units** or when you create the channel.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x1178)
        return val

    @co_pulse_freq.setter
    def co_pulse_freq(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x1178, val)

    @co_pulse_freq.deleter
    def co_pulse_freq(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1178)

    @property
    def co_pulse_freq_initial_delay(self):
        """
        float: Specifies in seconds the amount of time to wait before
            generating the first pulse.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x299)
        return val

    @co_pulse_freq_initial_delay.setter
    def co_pulse_freq_initial_delay(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x299, val)

    @co_pulse_freq_initial_delay.deleter
    def co_pulse_freq_initial_delay(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x299)

    @property
    def co_pulse_freq_units(self):
        """
        :class:`nidaqmx.constants.FrequencyUnits`: Specifies the units
            in which to define pulse frequency.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x18d5)
        return FrequencyUnits(val)

    @co_pulse_freq_units.setter
    def co_pulse_freq_units(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x18d5, val)

    @co_pulse_freq_units.deleter
    def co_pulse_freq_units(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x18d5)

    @property
    def co_pulse_high_ticks(self):
        """
        int: Specifies the number of ticks the pulse is high.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x1169)
        return val

    @co_pulse_high_ticks.setter
    def co_pulse_high_ticks(self, val):
        self._interpreter.set_chan_attribute_uint32(self._handle, self._name, 0x1169, val)

    @co_pulse_high_ticks.deleter
    def co_pulse_high_ticks(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1169)

    @property
    def co_pulse_high_time(self):
        """
        float: Specifies the amount of time that the pulse is at a high
            voltage. This value is in the units you specify with
            **co_pulse_time_units** or when you create the channel.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x18ba)
        return val

    @co_pulse_high_time.setter
    def co_pulse_high_time(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x18ba, val)

    @co_pulse_high_time.deleter
    def co_pulse_high_time(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x18ba)

    @property
    def co_pulse_idle_state(self):
        """
        :class:`nidaqmx.constants.Level`: Specifies the resting state of
            the output terminal.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x1170)
        return Level(val)

    @co_pulse_idle_state.setter
    def co_pulse_idle_state(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x1170, val)

    @co_pulse_idle_state.deleter
    def co_pulse_idle_state(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1170)

    @property
    def co_pulse_low_ticks(self):
        """
        int: Specifies the number of ticks the pulse is low.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x1171)
        return val

    @co_pulse_low_ticks.setter
    def co_pulse_low_ticks(self, val):
        self._interpreter.set_chan_attribute_uint32(self._handle, self._name, 0x1171, val)

    @co_pulse_low_ticks.deleter
    def co_pulse_low_ticks(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1171)

    @property
    def co_pulse_low_time(self):
        """
        float: Specifies the amount of time that the pulse is at a low
            voltage. This value is in the units you specify with
            **co_pulse_time_units** or when you create the channel.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x18bb)
        return val

    @co_pulse_low_time.setter
    def co_pulse_low_time(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x18bb, val)

    @co_pulse_low_time.deleter
    def co_pulse_low_time(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x18bb)

    @property
    def co_pulse_term(self):
        """
        str: Specifies on which terminal to generate pulses.
        """

        val = self._interpreter.get_chan_attribute_string(self._handle, self._name, 0x18e1)
        return val

    @co_pulse_term.setter
    def co_pulse_term(self, val):
        self._interpreter.set_chan_attribute_string(self._handle, self._name, 0x18e1, val)

    @co_pulse_term.deleter
    def co_pulse_term(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x18e1)

    @property
    def co_pulse_ticks_initial_delay(self):
        """
        int: Specifies the number of ticks to wait before generating the
            first pulse.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x298)
        return val

    @co_pulse_ticks_initial_delay.setter
    def co_pulse_ticks_initial_delay(self, val):
        self._interpreter.set_chan_attribute_uint32(self._handle, self._name, 0x298, val)

    @co_pulse_ticks_initial_delay.deleter
    def co_pulse_ticks_initial_delay(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x298)

    @property
    def co_pulse_time_initial_delay(self):
        """
        float: Specifies in seconds the amount of time to wait before
            generating the first pulse.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x18bc)
        return val

    @co_pulse_time_initial_delay.setter
    def co_pulse_time_initial_delay(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x18bc, val)

    @co_pulse_time_initial_delay.deleter
    def co_pulse_time_initial_delay(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x18bc)

    @property
    def co_pulse_time_units(self):
        """
        :class:`nidaqmx.constants.TimeUnits`: Specifies the units in
            which to define high and low pulse time.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x18d6)
        return TimeUnits(val)

    @co_pulse_time_units.setter
    def co_pulse_time_units(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x18d6, val)

    @co_pulse_time_units.deleter
    def co_pulse_time_units(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x18d6)

    @property
    def co_rdy_for_new_val(self):
        """
        bool: Indicates whether the counter is ready for new continuous
            pulse train values.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x22ff)
        return val

    @property
    def co_usb_xfer_req_count(self):
        """
        int: Specifies the maximum number of simultaneous USB transfers
            used to stream data. Modify this value to affect performance
            under different combinations of operating system and device.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x3005)
        return val

    @co_usb_xfer_req_count.setter
    def co_usb_xfer_req_count(self, val):
        self._interpreter.set_chan_attribute_uint32(self._handle, self._name, 0x3005, val)

    @co_usb_xfer_req_count.deleter
    def co_usb_xfer_req_count(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x3005)

    @property
    def co_usb_xfer_req_size(self):
        """
        int: Specifies the maximum size of a USB transfer request in
            bytes. Modify this value to affect performance under
            different combinations of operating system and device.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x2a93)
        return val

    @co_usb_xfer_req_size.setter
    def co_usb_xfer_req_size(self, val):
        self._interpreter.set_chan_attribute_uint32(self._handle, self._name, 0x2a93, val)

    @co_usb_xfer_req_size.deleter
    def co_usb_xfer_req_size(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2a93)

    @property
    def co_use_only_on_brd_mem(self):
        """
        bool: Specifies whether to write samples directly to the onboard
            memory of the device, bypassing the memory buffer.
            Generally, you cannot update onboard memory directly after
            you start the task. Onboard memory includes data FIFOs.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x2ecb)
        return val

    @co_use_only_on_brd_mem.setter
    def co_use_only_on_brd_mem(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x2ecb, val)

    @co_use_only_on_brd_mem.deleter
    def co_use_only_on_brd_mem(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2ecb)




# Do not edit this file; it was automatically generated.

from nidaqmx.constants import (
    DeassertCondition, DigitalWidthUnits, ExportAction, Level, Polarity,
    Signal)


class ExportSignals:
    """
    Represents the exported signal configurations for a DAQmx task.
    """
    def __init__(self, task_handle, interpreter):
        self._handle = task_handle
        self._interpreter = interpreter

    @property
    def adv_cmplt_event_delay(self):
        """
        float: Specifies the output signal delay in periods of the
            sample clock.
        """

        val = self._interpreter.get_exported_signal_attribute_double(self._handle, 0x1757)
        return val

    @adv_cmplt_event_delay.setter
    def adv_cmplt_event_delay(self, val):
        self._interpreter.set_exported_signal_attribute_double(self._handle, 0x1757, val)

    @adv_cmplt_event_delay.deleter
    def adv_cmplt_event_delay(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1757)

    @property
    def adv_cmplt_event_output_term(self):
        """
        str: Specifies the terminal to which to route the Advance
            Complete Event.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x1651)
        return val

    @adv_cmplt_event_output_term.setter
    def adv_cmplt_event_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x1651, val)

    @adv_cmplt_event_output_term.deleter
    def adv_cmplt_event_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1651)

    @property
    def adv_cmplt_event_pulse_polarity(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the polarity of
            the exported Advance Complete Event.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x1652)
        return Polarity(val)

    @adv_cmplt_event_pulse_polarity.setter
    def adv_cmplt_event_pulse_polarity(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x1652, val)

    @adv_cmplt_event_pulse_polarity.deleter
    def adv_cmplt_event_pulse_polarity(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1652)

    @property
    def adv_cmplt_event_pulse_width(self):
        """
        float: Specifies the width of the exported Advance Complete
            Event pulse.
        """

        val = self._interpreter.get_exported_signal_attribute_double(self._handle, 0x1654)
        return val

    @adv_cmplt_event_pulse_width.setter
    def adv_cmplt_event_pulse_width(self, val):
        self._interpreter.set_exported_signal_attribute_double(self._handle, 0x1654, val)

    @adv_cmplt_event_pulse_width.deleter
    def adv_cmplt_event_pulse_width(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1654)

    @property
    def adv_trig_output_term(self):
        """
        str: Specifies the terminal to which to route the Advance
            Trigger.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x1645)
        return val

    @adv_trig_output_term.setter
    def adv_trig_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x1645, val)

    @adv_trig_output_term.deleter
    def adv_trig_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1645)

    @property
    def adv_trig_pulse_polarity(self):
        """
        :class:`nidaqmx.constants.Polarity`: Indicates the polarity of
            the exported Advance Trigger.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x1646)
        return Polarity(val)

    @property
    def adv_trig_pulse_width(self):
        """
        float: Specifies the width of an exported Advance Trigger pulse.
            Specify this value in the units you specify with
            **adv_trig_pulse_width_units**.
        """

        val = self._interpreter.get_exported_signal_attribute_double(self._handle, 0x1648)
        return val

    @adv_trig_pulse_width.setter
    def adv_trig_pulse_width(self, val):
        self._interpreter.set_exported_signal_attribute_double(self._handle, 0x1648, val)

    @adv_trig_pulse_width.deleter
    def adv_trig_pulse_width(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1648)

    @property
    def adv_trig_pulse_width_units(self):
        """
        :class:`nidaqmx.constants.DigitalWidthUnits`: Specifies the
            units of **adv_trig_pulse_width**.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x1647)
        return DigitalWidthUnits(val)

    @adv_trig_pulse_width_units.setter
    def adv_trig_pulse_width_units(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x1647, val)

    @adv_trig_pulse_width_units.deleter
    def adv_trig_pulse_width_units(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1647)

    @property
    def ai_conv_clk_output_term(self):
        """
        str: Specifies the terminal to which to route the AI Convert
            Clock.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x1687)
        return val

    @ai_conv_clk_output_term.setter
    def ai_conv_clk_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x1687, val)

    @ai_conv_clk_output_term.deleter
    def ai_conv_clk_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1687)

    @property
    def ai_conv_clk_pulse_polarity(self):
        """
        :class:`nidaqmx.constants.Polarity`: Indicates the polarity of
            the exported AI Convert Clock. The polarity is fixed and
            independent of the active edge of the source of the AI
            Convert Clock.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x1688)
        return Polarity(val)

    @property
    def ai_hold_cmplt_event_output_term(self):
        """
        str: Specifies the terminal to which to route the AI Hold
            Complete Event.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x18ed)
        return val

    @ai_hold_cmplt_event_output_term.setter
    def ai_hold_cmplt_event_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x18ed, val)

    @ai_hold_cmplt_event_output_term.deleter
    def ai_hold_cmplt_event_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x18ed)

    @property
    def ai_hold_cmplt_event_pulse_polarity(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the polarity of
            an exported AI Hold Complete Event pulse.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x18ee)
        return Polarity(val)

    @ai_hold_cmplt_event_pulse_polarity.setter
    def ai_hold_cmplt_event_pulse_polarity(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x18ee, val)

    @ai_hold_cmplt_event_pulse_polarity.deleter
    def ai_hold_cmplt_event_pulse_polarity(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x18ee)

    @property
    def change_detect_event_output_term(self):
        """
        str: Specifies the terminal to which to route the Change
            Detection Event.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x2197)
        return val

    @change_detect_event_output_term.setter
    def change_detect_event_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x2197, val)

    @change_detect_event_output_term.deleter
    def change_detect_event_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x2197)

    @property
    def change_detect_event_pulse_polarity(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the polarity of
            an exported Change Detection Event pulse.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x2303)
        return Polarity(val)

    @change_detect_event_pulse_polarity.setter
    def change_detect_event_pulse_polarity(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x2303, val)

    @change_detect_event_pulse_polarity.deleter
    def change_detect_event_pulse_polarity(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x2303)

    @property
    def ctr_out_event_output_behavior(self):
        """
        :class:`nidaqmx.constants.ExportAction`: Specifies whether the
            exported Counter Output Event pulses or changes from one
            state to the other when the counter reaches terminal count.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x174f)
        return ExportAction(val)

    @ctr_out_event_output_behavior.setter
    def ctr_out_event_output_behavior(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x174f, val)

    @ctr_out_event_output_behavior.deleter
    def ctr_out_event_output_behavior(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x174f)

    @property
    def ctr_out_event_output_term(self):
        """
        str: Specifies the terminal to which to route the Counter Output
            Event.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x1717)
        return val

    @ctr_out_event_output_term.setter
    def ctr_out_event_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x1717, val)

    @ctr_out_event_output_term.deleter
    def ctr_out_event_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1717)

    @property
    def ctr_out_event_pulse_polarity(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the polarity of
            the pulses at the output terminal of the counter when
            **ctr_out_event_output_behavior** is
            **ExportActions2.PULSE**. NI-DAQmx ignores this property if
            **ctr_out_event_output_behavior** is
            **ExportActions2.TOGGLE**.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x1718)
        return Polarity(val)

    @ctr_out_event_pulse_polarity.setter
    def ctr_out_event_pulse_polarity(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x1718, val)

    @ctr_out_event_pulse_polarity.deleter
    def ctr_out_event_pulse_polarity(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1718)

    @property
    def ctr_out_event_toggle_idle_state(self):
        """
        :class:`nidaqmx.constants.Level`: Specifies the initial state of
            the output terminal of the counter when
            **ctr_out_event_output_behavior** is
            **ExportActions2.TOGGLE**. The terminal enters this state
            when NI-DAQmx commits the task.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x186a)
        return Level(val)

    @ctr_out_event_toggle_idle_state.setter
    def ctr_out_event_toggle_idle_state(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x186a, val)

    @ctr_out_event_toggle_idle_state.deleter
    def ctr_out_event_toggle_idle_state(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x186a)

    @property
    def data_active_event_lvl_active_lvl(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the polarity of
            the exported Data Active Event.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x1634)
        return Polarity(val)

    @data_active_event_lvl_active_lvl.setter
    def data_active_event_lvl_active_lvl(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x1634, val)

    @data_active_event_lvl_active_lvl.deleter
    def data_active_event_lvl_active_lvl(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1634)

    @property
    def data_active_event_output_term(self):
        """
        str: Specifies the terminal to which to export the Data Active
            Event.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x1633)
        return val

    @data_active_event_output_term.setter
    def data_active_event_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x1633, val)

    @data_active_event_output_term.deleter
    def data_active_event_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1633)

    @property
    def divided_samp_clk_timebase_output_term(self):
        """
        str: Specifies the terminal to which to route the Divided Sample
            Clock Timebase.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x21a1)
        return val

    @divided_samp_clk_timebase_output_term.setter
    def divided_samp_clk_timebase_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x21a1, val)

    @divided_samp_clk_timebase_output_term.deleter
    def divided_samp_clk_timebase_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x21a1)

    @property
    def exported_10_mhz_ref_clk_output_term(self):
        """
        str: Specifies the terminal to which to route the 10MHz Clock.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x226e)
        return val

    @exported_10_mhz_ref_clk_output_term.setter
    def exported_10_mhz_ref_clk_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x226e, val)

    @exported_10_mhz_ref_clk_output_term.deleter
    def exported_10_mhz_ref_clk_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x226e)

    @property
    def exported_20_mhz_timebase_output_term(self):
        """
        str: Specifies the terminal to which to route the 20MHz
            Timebase.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x1657)
        return val

    @exported_20_mhz_timebase_output_term.setter
    def exported_20_mhz_timebase_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x1657, val)

    @exported_20_mhz_timebase_output_term.deleter
    def exported_20_mhz_timebase_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1657)

    @property
    def hshk_event_delay(self):
        """
        float: Specifies the number of seconds to delay after the
            Handshake Trigger deasserts before asserting the Handshake
            Event.
        """

        val = self._interpreter.get_exported_signal_attribute_double(self._handle, 0x22bc)
        return val

    @hshk_event_delay.setter
    def hshk_event_delay(self, val):
        self._interpreter.set_exported_signal_attribute_double(self._handle, 0x22bc, val)

    @hshk_event_delay.deleter
    def hshk_event_delay(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x22bc)

    @property
    def hshk_event_interlocked_assert_on_start(self):
        """
        bool: Specifies to assert the Handshake Event when the task
            starts if **hshk_event_output_behavior** is
            **ExportActions5.INTERLOCKED**.
        """

        val = self._interpreter.get_exported_signal_attribute_bool(self._handle, 0x22be)
        return val

    @hshk_event_interlocked_assert_on_start.setter
    def hshk_event_interlocked_assert_on_start(self, val):
        self._interpreter.set_exported_signal_attribute_bool(self._handle, 0x22be, val)

    @hshk_event_interlocked_assert_on_start.deleter
    def hshk_event_interlocked_assert_on_start(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x22be)

    @property
    def hshk_event_interlocked_asserted_lvl(self):
        """
        :class:`nidaqmx.constants.Level`: Specifies the asserted level
            of the exported Handshake Event if
            **hshk_event_output_behavior** is
            **ExportActions5.INTERLOCKED**.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x22bd)
        return Level(val)

    @hshk_event_interlocked_asserted_lvl.setter
    def hshk_event_interlocked_asserted_lvl(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x22bd, val)

    @hshk_event_interlocked_asserted_lvl.deleter
    def hshk_event_interlocked_asserted_lvl(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x22bd)

    @property
    def hshk_event_interlocked_deassert_delay(self):
        """
        float: Specifies in seconds the amount of time to wait after the
            Handshake Trigger asserts before deasserting the Handshake
            Event if **hshk_event_output_behavior** is
            **ExportActions5.INTERLOCKED**.
        """

        val = self._interpreter.get_exported_signal_attribute_double(self._handle, 0x22bf)
        return val

    @hshk_event_interlocked_deassert_delay.setter
    def hshk_event_interlocked_deassert_delay(self, val):
        self._interpreter.set_exported_signal_attribute_double(self._handle, 0x22bf, val)

    @hshk_event_interlocked_deassert_delay.deleter
    def hshk_event_interlocked_deassert_delay(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x22bf)

    @property
    def hshk_event_output_behavior(self):
        """
        :class:`nidaqmx.constants.ExportAction`: Specifies the output
            behavior of the Handshake Event.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x22bb)
        return ExportAction(val)

    @hshk_event_output_behavior.setter
    def hshk_event_output_behavior(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x22bb, val)

    @hshk_event_output_behavior.deleter
    def hshk_event_output_behavior(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x22bb)

    @property
    def hshk_event_output_term(self):
        """
        str: Specifies the terminal to which to route the Handshake
            Event.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x22ba)
        return val

    @hshk_event_output_term.setter
    def hshk_event_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x22ba, val)

    @hshk_event_output_term.deleter
    def hshk_event_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x22ba)

    @property
    def hshk_event_pulse_polarity(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the polarity of
            the exported Handshake Event if
            **hshk_event_output_behavior** is **ExportActions5.PULSE**.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x22c0)
        return Polarity(val)

    @hshk_event_pulse_polarity.setter
    def hshk_event_pulse_polarity(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x22c0, val)

    @hshk_event_pulse_polarity.deleter
    def hshk_event_pulse_polarity(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x22c0)

    @property
    def hshk_event_pulse_width(self):
        """
        float: Specifies in seconds the pulse width of the exported
            Handshake Event if **hshk_event_output_behavior** is
            **ExportActions5.PULSE**.
        """

        val = self._interpreter.get_exported_signal_attribute_double(self._handle, 0x22c1)
        return val

    @hshk_event_pulse_width.setter
    def hshk_event_pulse_width(self, val):
        self._interpreter.set_exported_signal_attribute_double(self._handle, 0x22c1, val)

    @hshk_event_pulse_width.deleter
    def hshk_event_pulse_width(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x22c1)

    @property
    def pause_trig_lvl_active_lvl(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the active level
            of the exported Pause Trigger.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x1616)
        return Polarity(val)

    @pause_trig_lvl_active_lvl.setter
    def pause_trig_lvl_active_lvl(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x1616, val)

    @pause_trig_lvl_active_lvl.deleter
    def pause_trig_lvl_active_lvl(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1616)

    @property
    def pause_trig_output_term(self):
        """
        str: Specifies the terminal to which to route the Pause Trigger.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x1615)
        return val

    @pause_trig_output_term.setter
    def pause_trig_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x1615, val)

    @pause_trig_output_term.deleter
    def pause_trig_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1615)

    @property
    def rdy_for_start_event_lvl_active_lvl(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the polarity of
            the exported Ready for Start Event.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x1751)
        return Polarity(val)

    @rdy_for_start_event_lvl_active_lvl.setter
    def rdy_for_start_event_lvl_active_lvl(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x1751, val)

    @rdy_for_start_event_lvl_active_lvl.deleter
    def rdy_for_start_event_lvl_active_lvl(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1751)

    @property
    def rdy_for_start_event_output_term(self):
        """
        str: Specifies the terminal to which to route the Ready for
            Start Event.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x1609)
        return val

    @rdy_for_start_event_output_term.setter
    def rdy_for_start_event_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x1609, val)

    @rdy_for_start_event_output_term.deleter
    def rdy_for_start_event_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1609)

    @property
    def rdy_for_xfer_event_deassert_cond(self):
        """
        :class:`nidaqmx.constants.DeassertCondition`: Specifies when the
            ready for transfer event deasserts.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x2963)
        return DeassertCondition(val)

    @rdy_for_xfer_event_deassert_cond.setter
    def rdy_for_xfer_event_deassert_cond(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x2963, val)

    @rdy_for_xfer_event_deassert_cond.deleter
    def rdy_for_xfer_event_deassert_cond(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x2963)

    @property
    def rdy_for_xfer_event_deassert_cond_custom_threshold(self):
        """
        int: Specifies in samples the threshold below which the Ready
            for Transfer Event deasserts. This threshold is an amount of
            space available in the onboard memory of the device.
            **rdy_for_xfer_event_deassert_cond** must be
            **DeassertCondition.ONBOARD_MEMORY_CUSTOM_THRESHOLD** to use
            a custom threshold.
        """

        val = self._interpreter.get_exported_signal_attribute_uint32(self._handle, 0x2964)
        return val

    @rdy_for_xfer_event_deassert_cond_custom_threshold.setter
    def rdy_for_xfer_event_deassert_cond_custom_threshold(self, val):
        self._interpreter.set_exported_signal_attribute_uint32(self._handle, 0x2964, val)

    @rdy_for_xfer_event_deassert_cond_custom_threshold.deleter
    def rdy_for_xfer_event_deassert_cond_custom_threshold(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x2964)

    @property
    def rdy_for_xfer_event_lvl_active_lvl(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the active level
            of the exported Ready for Transfer Event.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x22b6)
        return Polarity(val)

    @rdy_for_xfer_event_lvl_active_lvl.setter
    def rdy_for_xfer_event_lvl_active_lvl(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x22b6, val)

    @rdy_for_xfer_event_lvl_active_lvl.deleter
    def rdy_for_xfer_event_lvl_active_lvl(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x22b6)

    @property
    def rdy_for_xfer_event_output_term(self):
        """
        str: Specifies the terminal to which to route the Ready for
            Transfer Event.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x22b5)
        return val

    @rdy_for_xfer_event_output_term.setter
    def rdy_for_xfer_event_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x22b5, val)

    @rdy_for_xfer_event_output_term.deleter
    def rdy_for_xfer_event_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x22b5)

    @property
    def ref_trig_output_term(self):
        """
        str: Specifies the terminal to which to route the Reference
            Trigger.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x590)
        return val

    @ref_trig_output_term.setter
    def ref_trig_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x590, val)

    @ref_trig_output_term.deleter
    def ref_trig_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x590)

    @property
    def ref_trig_pulse_polarity(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the polarity of
            the exported Reference Trigger.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x591)
        return Polarity(val)

    @ref_trig_pulse_polarity.setter
    def ref_trig_pulse_polarity(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x591, val)

    @ref_trig_pulse_polarity.deleter
    def ref_trig_pulse_polarity(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x591)

    @property
    def samp_clk_delay_offset(self):
        """
        float: Specifies in seconds the amount of time to offset the
            exported Sample clock.  Refer to timing diagrams for
            generation applications in the device documentation for more
            information about this value.
        """

        val = self._interpreter.get_exported_signal_attribute_double(self._handle, 0x21c4)
        return val

    @samp_clk_delay_offset.setter
    def samp_clk_delay_offset(self, val):
        self._interpreter.set_exported_signal_attribute_double(self._handle, 0x21c4, val)

    @samp_clk_delay_offset.deleter
    def samp_clk_delay_offset(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x21c4)

    @property
    def samp_clk_output_behavior(self):
        """
        :class:`nidaqmx.constants.ExportAction`: Specifies whether the
            exported Sample Clock issues a pulse at the beginning of a
            sample or changes to a high state for the duration of the
            sample.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x186b)
        return ExportAction(val)

    @samp_clk_output_behavior.setter
    def samp_clk_output_behavior(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x186b, val)

    @samp_clk_output_behavior.deleter
    def samp_clk_output_behavior(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x186b)

    @property
    def samp_clk_output_term(self):
        """
        str: Specifies the terminal to which to route the Sample Clock.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x1663)
        return val

    @samp_clk_output_term.setter
    def samp_clk_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x1663, val)

    @samp_clk_output_term.deleter
    def samp_clk_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1663)

    @property
    def samp_clk_pulse_polarity(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the polarity of
            the exported Sample Clock if **samp_clk_output_behavior** is
            **ExportActions3.PULSE**.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x1664)
        return Polarity(val)

    @samp_clk_pulse_polarity.setter
    def samp_clk_pulse_polarity(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x1664, val)

    @samp_clk_pulse_polarity.deleter
    def samp_clk_pulse_polarity(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x1664)

    @property
    def samp_clk_timebase_output_term(self):
        """
        str: Specifies the terminal to which to route the Sample Clock
            Timebase.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x18f9)
        return val

    @samp_clk_timebase_output_term.setter
    def samp_clk_timebase_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x18f9, val)

    @samp_clk_timebase_output_term.deleter
    def samp_clk_timebase_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x18f9)

    @property
    def start_trig_output_term(self):
        """
        str: Specifies the terminal to which to route the Start Trigger.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x584)
        return val

    @start_trig_output_term.setter
    def start_trig_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x584, val)

    @start_trig_output_term.deleter
    def start_trig_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x584)

    @property
    def start_trig_pulse_polarity(self):
        """
        :class:`nidaqmx.constants.Polarity`: Specifies the polarity of
            the exported Start Trigger.
        """

        val = self._interpreter.get_exported_signal_attribute_int32(self._handle, 0x585)
        return Polarity(val)

    @start_trig_pulse_polarity.setter
    def start_trig_pulse_polarity(self, val):
        val = val.value
        self._interpreter.set_exported_signal_attribute_int32(self._handle, 0x585, val)

    @start_trig_pulse_polarity.deleter
    def start_trig_pulse_polarity(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x585)

    @property
    def sync_pulse_event_output_term(self):
        """
        str: Specifies the terminal to which to route the
            Synchronization Pulse Event.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x223c)
        return val

    @sync_pulse_event_output_term.setter
    def sync_pulse_event_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x223c, val)

    @sync_pulse_event_output_term.deleter
    def sync_pulse_event_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x223c)

    @property
    def watchdog_expired_event_output_term(self):
        """
        str: Specifies the terminal  to which to route the Watchdog
            Timer Expired Event.
        """

        val = self._interpreter.get_exported_signal_attribute_string(self._handle, 0x21aa)
        return val

    @watchdog_expired_event_output_term.setter
    def watchdog_expired_event_output_term(self, val):
        self._interpreter.set_exported_signal_attribute_string(self._handle, 0x21aa, val)

    @watchdog_expired_event_output_term.deleter
    def watchdog_expired_event_output_term(self):
        self._interpreter.reset_exported_signal_attribute(self._handle, 0x21aa)

    def export_signal(self, signal_id, output_terminal):
        """
        Routes a control signal to the terminal you specify. The output
        terminal can reside on the device that generates the control
        signal or on a different device. You can use this function to
        share clocks and triggers among multiple tasks and devices. The
        routes this function creates are task-based routes.

        Args:
            signal_id (nidaqmx.constants.Signal): Is the name of the
                trigger, clock, or event to export.
            output_terminal (str): Is the destination of the exported
                signal. A DAQmx terminal constant lists all terminals on
                installed devices. You can also specify a string
                containing a comma-delimited list of terminal names.
        """

        self._interpreter.export_signal(
            self._handle, signal_id.value, output_terminal)





from nidaqmx.task.collections._channel_collection import ChannelCollection
from nidaqmx.task.collections._ai_channel_collection import AIChannelCollection
from nidaqmx.task.collections._ao_channel_collection import AOChannelCollection
from nidaqmx.task.collections._ci_channel_collection import CIChannelCollection
from nidaqmx.task.collections._co_channel_collection import COChannelCollection
from nidaqmx.task.collections._di_channel_collection import DIChannelCollection
from nidaqmx.task.collections._do_channel_collection import DOChannelCollection

__all__ = ['ChannelCollection', 'AIChannelCollection', 'AOChannelCollection', 'CIChannelCollection', 'COChannelCollection', 'DIChannelCollection', 'DOChannelCollection',]


# Do not edit this file; it was automatically generated.

import deprecation

from nidaqmx.constants import (
    Level, WatchdogAOExpirState, WatchdogCOExpirState)


class ExpirationState:
    """
    Represents a DAQmx Watchdog expiration state.
    """
    def __init__(self, task_handle, physical_channel, interpreter):
        self._handle = task_handle
        self._physical_channel = physical_channel        
        self._interpreter = interpreter

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return (self._handle == other._handle and
                    self._physical_channel == other._physical_channel)
        return False

    def __hash__(self):
        return self._interpreter.hash_task_handle(self._handle) ^ hash(self._physical_channel)

    def __ne__(self, other):
        return not self.__eq__(other)

    @property
    def ao_output_type(self):
        """
        :class:`nidaqmx.constants.WatchdogAOExpirState`: Specifies the
            output type of the analog output physical channels when the
            watchdog task expires.
        """

        val = self._interpreter.get_watchdog_attribute_int32(self._handle, self._physical_channel, 0x305e)
        return WatchdogAOExpirState(val)

    @ao_output_type.setter
    def ao_output_type(self, val):
        val = val.value
        self._interpreter.set_watchdog_attribute_int32(self._handle, self._physical_channel, 0x305e, val)

    @ao_output_type.deleter
    def ao_output_type(self):
        self._interpreter.reset_watchdog_attribute(self._handle, self._physical_channel, 0x305e)

    @property
    def ao_state(self):
        """
        float: Specifies the state to set the analog output physical
            channels when the watchdog task expires.
        """

        val = self._interpreter.get_watchdog_attribute_double(self._handle, self._physical_channel, 0x305f)
        return val

    @ao_state.setter
    def ao_state(self, val):
        self._interpreter.set_watchdog_attribute_double(self._handle, self._physical_channel, 0x305f, val)

    @ao_state.deleter
    def ao_state(self):
        self._interpreter.reset_watchdog_attribute(self._handle, self._physical_channel, 0x305f)

    @property
    def co_state(self):
        """
        :class:`nidaqmx.constants.WatchdogCOExpirState`: Specifies the
            state to set the counter output channel terminal when the
            watchdog task expires.
        """

        val = self._interpreter.get_watchdog_attribute_int32(self._handle, self._physical_channel, 0x3060)
        return WatchdogCOExpirState(val)

    @co_state.setter
    def co_state(self, val):
        val = val.value
        self._interpreter.set_watchdog_attribute_int32(self._handle, self._physical_channel, 0x3060, val)

    @co_state.deleter
    def co_state(self):
        self._interpreter.reset_watchdog_attribute(self._handle, self._physical_channel, 0x3060)

    @property
    def do_state(self):
        """
        :class:`nidaqmx.constants.Level`: Specifies the state to which
            to set the digital physical channels when the watchdog task
            expires.  You cannot modify the expiration state of
            dedicated digital input physical channels.
        """

        val = self._interpreter.get_watchdog_attribute_int32(self._handle, self._physical_channel, 0x21a7)
        return Level(val)

    @do_state.setter
    def do_state(self, val):
        val = val.value
        self._interpreter.set_watchdog_attribute_int32(self._handle, self._physical_channel, 0x21a7, val)

    @do_state.deleter
    def do_state(self):
        self._interpreter.reset_watchdog_attribute(self._handle, self._physical_channel, 0x21a7)

    @property
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use ao_output_type instead.")
    def expir_states_ao_type(self):
        return self.ao_output_type

    @expir_states_ao_type.setter
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use ao_output_type instead.")
    def expir_states_ao_type(self, val):
        self.ao_output_type = val

    @expir_states_ao_type.deleter
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use ao_output_type instead.")
    def expir_states_ao_type(self):
        del self.ao_output_type

    @property
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use co_state instead.")
    def expir_states_co_state(self):
        return self.co_state

    @expir_states_co_state.setter
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use co_state instead.")
    def expir_states_co_state(self, val):
        self.co_state = val

    @expir_states_co_state.deleter
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use co_state instead.")
    def expir_states_co_state(self):
        del self.co_state

    @property
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use do_state instead.")
    def expir_states_do_state(self):
        return self.do_state

    @expir_states_do_state.setter
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use do_state instead.")
    def expir_states_do_state(self, val):
        self.do_state = val

    @expir_states_do_state.deleter
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use do_state instead.")
    def expir_states_do_state(self):
        del self.do_state

    @property
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use ao_state instead.")
    def expir_states_ao_state(self):
        return self.ao_state

    @expir_states_ao_state.setter
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use ao_state instead.")
    def expir_states_ao_state(self, val):
        self.ao_state = val

    @expir_states_ao_state.deleter
    @deprecation.deprecated(deprecated_in="0.7.0", details="Use ao_state instead.")
    def expir_states_ao_state(self):
        del self.ao_state




# Do not edit this file; it was automatically generated.
import abc
from typing import Optional


class BaseEventHandler(abc.ABC):
    """Interpreter-specific object that is returned from register_*_event()."""
    __slots__ = ()

    @abc.abstractmethod
    def close(self) -> None:
        """Release resources used by the event handler.

        After releasing the resources, this method may report handler-specific errors
        (e.g. gRPC stream errors) by raising an exception.
        """
        raise NotImplementedError


class BaseInterpreter(abc.ABC):
    """
    Contains signature of functions for all DAQmx APIs.
    """
    __slots__ = ()

    @abc.abstractmethod
    def add_cdaq_sync_connection(self, port_list):
        raise NotImplementedError

    @abc.abstractmethod
    def add_global_chans_to_task(self, task, channel_names):
        raise NotImplementedError

    @abc.abstractmethod
    def add_network_device(
            self, ip_address, device_name, attempt_reservation, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def are_configured_cdaq_sync_ports_disconnected(
            self, chassis_devices_ports, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def auto_configure_cdaq_sync_connections(
            self, chassis_devices_ports, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def calculate_reverse_poly_coeff(
            self, forward_coeffs, min_val_x, max_val_x, num_points_to_compute,
            reverse_poly_order):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_anlg_edge_ref_trig(
            self, task, trigger_source, pretrigger_samples, trigger_slope,
            trigger_level):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_anlg_edge_start_trig(
            self, task, trigger_source, trigger_slope, trigger_level):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_anlg_multi_edge_ref_trig(
            self, task, trigger_sources, pretrigger_samples,
            trigger_slope_array, trigger_level_array):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_anlg_multi_edge_start_trig(
            self, task, trigger_sources, trigger_slope_array,
            trigger_level_array):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_anlg_window_ref_trig(
            self, task, trigger_source, window_top, window_bottom,
            pretrigger_samples, trigger_when):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_anlg_window_start_trig(
            self, task, window_top, window_bottom, trigger_source,
            trigger_when):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_burst_handshaking_timing_export_clock(
            self, task, sample_clk_rate, sample_clk_outp_term, sample_mode,
            samps_per_chan, sample_clk_pulse_polarity, pause_when,
            ready_event_active_level):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_burst_handshaking_timing_import_clock(
            self, task, sample_clk_rate, sample_clk_src, sample_mode,
            samps_per_chan, sample_clk_active_edge, pause_when,
            ready_event_active_level):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_change_detection_timing(
            self, task, rising_edge_chan, falling_edge_chan, sample_mode,
            samps_per_chan):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_dig_edge_ref_trig(
            self, task, trigger_source, pretrigger_samples, trigger_edge):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_dig_edge_start_trig(self, task, trigger_source, trigger_edge):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_dig_pattern_ref_trig(
            self, task, trigger_source, trigger_pattern, pretrigger_samples,
            trigger_when):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_dig_pattern_start_trig(
            self, task, trigger_source, trigger_pattern, trigger_when):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_handshaking_timing(self, task, sample_mode, samps_per_chan):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_implicit_timing(self, task, sample_mode, samps_per_chan):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_pipelined_samp_clk_timing(
            self, task, rate, source, active_edge, sample_mode,
            samps_per_chan):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_samp_clk_timing(
            self, task, rate, source, active_edge, sample_mode,
            samps_per_chan):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_time_start_trig(self, task, when, timescale):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_watchdog_ao_expir_states(
            self, task, channel_names, expir_state_array, output_type_array):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_watchdog_co_expir_states(
            self, task, channel_names, expir_state_array):
        raise NotImplementedError

    @abc.abstractmethod
    def cfg_watchdog_do_expir_states(
            self, task, channel_names, expir_state_array):
        raise NotImplementedError

    @abc.abstractmethod
    def clear_task(self, task):
        raise NotImplementedError

    @abc.abstractmethod
    def clear_teds(self, physical_channel):
        raise NotImplementedError

    @abc.abstractmethod
    def configure_logging(
            self, task, file_path, logging_mode, group_name, operation):
        raise NotImplementedError

    @abc.abstractmethod
    def configure_teds(self, physical_channel, file_path):
        raise NotImplementedError

    @abc.abstractmethod
    def connect_terms(
            self, source_terminal, destination_terminal, signal_modifiers):
        raise NotImplementedError

    @abc.abstractmethod
    def control_watchdog_task(self, task, action):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_accel4_wire_dc_voltage_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, sensitivity,
            sensitivity_units, voltage_excit_source, voltage_excit_val,
            use_excit_for_scaling, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_accel_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, sensitivity,
            sensitivity_units, current_excit_source, current_excit_val,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_accel_charge_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, sensitivity,
            sensitivity_units, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_bridge_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, bridge_config, voltage_excit_source,
            voltage_excit_val, nominal_bridge_resistance, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_charge_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_current_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, shunt_resistor_loc,
            ext_shunt_resistor_val, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_current_rms_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, shunt_resistor_loc,
            ext_shunt_resistor_val, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_force_bridge_polynomial_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, bridge_config, voltage_excit_source,
            voltage_excit_val, nominal_bridge_resistance, forward_coeffs,
            reverse_coeffs, electrical_units, physical_units,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_force_bridge_table_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, bridge_config, voltage_excit_source,
            voltage_excit_val, nominal_bridge_resistance, electrical_vals,
            electrical_units, physical_vals, physical_units,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_force_bridge_two_point_lin_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, bridge_config, voltage_excit_source,
            voltage_excit_val, nominal_bridge_resistance,
            first_electrical_val, second_electrical_val, electrical_units,
            first_physical_val, second_physical_val, physical_units,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_force_iepe_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, sensitivity,
            sensitivity_units, current_excit_source, current_excit_val,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_freq_voltage_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, threshold_level, hysteresis, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_microphone_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, units, mic_sensitivity, max_snd_press_level,
            current_excit_source, current_excit_val, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_pos_eddy_curr_prox_probe_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, sensitivity, sensitivity_units,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_pos_lvdt_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, sensitivity, sensitivity_units,
            voltage_excit_source, voltage_excit_val, voltage_excit_freq,
            ac_excit_wire_mode, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_pos_rvdt_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, sensitivity, sensitivity_units,
            voltage_excit_source, voltage_excit_val, voltage_excit_freq,
            ac_excit_wire_mode, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_power_chan(
            self, task, physical_channel, voltage_setpoint, current_setpoint,
            output_enable, name_to_assign_to_channel):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_pressure_bridge_polynomial_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, bridge_config, voltage_excit_source,
            voltage_excit_val, nominal_bridge_resistance, forward_coeffs,
            reverse_coeffs, electrical_units, physical_units,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_pressure_bridge_table_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, bridge_config, voltage_excit_source,
            voltage_excit_val, nominal_bridge_resistance, electrical_vals,
            electrical_units, physical_vals, physical_units,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_pressure_bridge_two_point_lin_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, bridge_config, voltage_excit_source,
            voltage_excit_val, nominal_bridge_resistance,
            first_electrical_val, second_electrical_val, electrical_units,
            first_physical_val, second_physical_val, physical_units,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_resistance_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, resistance_config, current_excit_source,
            current_excit_val, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_rosette_strain_gage_chan(
            self, task, physical_channel, rosette_type, gage_orientation,
            rosette_meas_types, name_to_assign_to_channel, min_val, max_val,
            strain_config, voltage_excit_source, voltage_excit_val,
            gage_factor, nominal_gage_resistance, poisson_ratio,
            lead_wire_resistance):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_strain_gage_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, strain_config, voltage_excit_source,
            voltage_excit_val, gage_factor, initial_bridge_voltage,
            nominal_gage_resistance, poisson_ratio, lead_wire_resistance,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_temp_built_in_sensor_chan(
            self, task, physical_channel, name_to_assign_to_channel, units):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_thrmcpl_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, thermocouple_type, cjc_source, cjc_val,
            cjc_channel):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_thrmstr_chan_iex(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, resistance_config, current_excit_source,
            current_excit_val, a, b, c):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_thrmstr_chan_vex(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, resistance_config, voltage_excit_source,
            voltage_excit_val, a, b, c, r_1):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_torque_bridge_polynomial_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, bridge_config, voltage_excit_source,
            voltage_excit_val, nominal_bridge_resistance, forward_coeffs,
            reverse_coeffs, electrical_units, physical_units,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_torque_bridge_table_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, bridge_config, voltage_excit_source,
            voltage_excit_val, nominal_bridge_resistance, electrical_vals,
            electrical_units, physical_vals, physical_units,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_torque_bridge_two_point_lin_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, bridge_config, voltage_excit_source,
            voltage_excit_val, nominal_bridge_resistance,
            first_electrical_val, second_electrical_val, electrical_units,
            first_physical_val, second_physical_val, physical_units,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_velocity_iepe_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, sensitivity,
            sensitivity_units, current_excit_source, current_excit_val,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_voltage_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_voltage_chan_with_excit(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, bridge_config,
            voltage_excit_source, voltage_excit_val, use_excit_for_scaling,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ai_voltage_rms_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_airtd_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, rtd_type, resistance_config, current_excit_source,
            current_excit_val, r_0):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ao_current_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ao_func_gen_chan(
            self, task, physical_channel, name_to_assign_to_channel, type,
            freq, amplitude, offset):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ao_voltage_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_ang_encoder_chan(
            self, task, counter, name_to_assign_to_channel, decoding_type,
            zidx_enable, zidx_val, zidx_phase, units, pulses_per_rev,
            initial_angle, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_ang_velocity_chan(
            self, task, counter, name_to_assign_to_channel, min_val, max_val,
            decoding_type, units, pulses_per_rev, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_count_edges_chan(
            self, task, counter, name_to_assign_to_channel, edge,
            initial_count, count_direction):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_duty_cycle_chan(
            self, task, counter, name_to_assign_to_channel, min_freq,
            max_freq, edge, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_freq_chan(
            self, task, counter, name_to_assign_to_channel, min_val, max_val,
            units, edge, meas_method, meas_time, divisor, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_lin_encoder_chan(
            self, task, counter, name_to_assign_to_channel, decoding_type,
            zidx_enable, zidx_val, zidx_phase, units, dist_per_pulse,
            initial_pos, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_lin_velocity_chan(
            self, task, counter, name_to_assign_to_channel, min_val, max_val,
            decoding_type, units, dist_per_pulse, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_period_chan(
            self, task, counter, name_to_assign_to_channel, min_val, max_val,
            units, edge, meas_method, meas_time, divisor, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_pulse_chan_freq(
            self, task, counter, name_to_assign_to_channel, min_val, max_val,
            units):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_pulse_chan_ticks(
            self, task, counter, name_to_assign_to_channel, source_terminal,
            min_val, max_val):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_pulse_chan_time(
            self, task, counter, name_to_assign_to_channel, min_val, max_val,
            units):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_pulse_width_chan(
            self, task, counter, name_to_assign_to_channel, min_val, max_val,
            units, starting_edge, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_semi_period_chan(
            self, task, counter, name_to_assign_to_channel, min_val, max_val,
            units, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_ci_two_edge_sep_chan(
            self, task, counter, name_to_assign_to_channel, min_val, max_val,
            units, first_edge, second_edge, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_cigps_timestamp_chan(
            self, task, counter, name_to_assign_to_channel, units,
            sync_method, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_co_pulse_chan_freq(
            self, task, counter, name_to_assign_to_channel, units, idle_state,
            initial_delay, freq, duty_cycle):
        raise NotImplementedError

    @abc.abstractmethod
    def create_co_pulse_chan_ticks(
            self, task, counter, source_terminal, name_to_assign_to_channel,
            idle_state, initial_delay, low_ticks, high_ticks):
        raise NotImplementedError

    @abc.abstractmethod
    def create_co_pulse_chan_time(
            self, task, counter, name_to_assign_to_channel, units, idle_state,
            initial_delay, low_time, high_time):
        raise NotImplementedError

    @abc.abstractmethod
    def create_di_chan(
            self, task, lines, name_to_assign_to_lines, line_grouping):
        raise NotImplementedError

    @abc.abstractmethod
    def create_do_chan(
            self, task, lines, name_to_assign_to_lines, line_grouping):
        raise NotImplementedError

    @abc.abstractmethod
    def create_lin_scale(
            self, name, slope, y_intercept, pre_scaled_units, scaled_units):
        raise NotImplementedError

    @abc.abstractmethod
    def create_map_scale(
            self, name, prescaled_min, prescaled_max, scaled_min, scaled_max,
            pre_scaled_units, scaled_units):
        raise NotImplementedError

    @abc.abstractmethod
    def create_polynomial_scale(
            self, name, forward_coeffs, reverse_coeffs, pre_scaled_units,
            scaled_units):
        raise NotImplementedError

    @abc.abstractmethod
    def create_table_scale(
            self, name, prescaled_vals, scaled_vals, pre_scaled_units,
            scaled_units):
        raise NotImplementedError

    @abc.abstractmethod
    def create_task(self, session_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_accel_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, current_excit_source,
            current_excit_val, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_bridge_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, voltage_excit_source, voltage_excit_val,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_current_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, shunt_resistor_loc,
            ext_shunt_resistor_val, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_force_bridge_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, voltage_excit_source, voltage_excit_val,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_force_iepe_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, current_excit_source,
            current_excit_val, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_microphone_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, units, max_snd_press_level, current_excit_source,
            current_excit_val, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_pos_lvdt_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, voltage_excit_source, voltage_excit_val,
            voltage_excit_freq, ac_excit_wire_mode, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_pos_rvdt_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, voltage_excit_source, voltage_excit_val,
            voltage_excit_freq, ac_excit_wire_mode, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_pressure_bridge_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, voltage_excit_source, voltage_excit_val,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_resistance_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, resistance_config, current_excit_source,
            current_excit_val, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_strain_gage_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, voltage_excit_source, voltage_excit_val,
            initial_bridge_voltage, lead_wire_resistance, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_thrmcpl_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, cjc_source, cjc_val, cjc_channel):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_thrmstr_chan_iex(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, resistance_config, current_excit_source,
            current_excit_val):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_thrmstr_chan_vex(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, resistance_config, voltage_excit_source,
            voltage_excit_val, r_1):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_torque_bridge_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, voltage_excit_source, voltage_excit_val,
            custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_voltage_chan(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsai_voltage_chan_with_excit(
            self, task, physical_channel, name_to_assign_to_channel,
            terminal_config, min_val, max_val, units, voltage_excit_source,
            voltage_excit_val, custom_scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def create_tedsairtd_chan(
            self, task, physical_channel, name_to_assign_to_channel, min_val,
            max_val, units, resistance_config, current_excit_source,
            current_excit_val):
        raise NotImplementedError

    @abc.abstractmethod
    def create_watchdog_timer_task_ex(
            self, device_name, session_name, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def delete_network_device(self, device_name):
        raise NotImplementedError

    @abc.abstractmethod
    def delete_saved_global_chan(self, channel_name):
        raise NotImplementedError

    @abc.abstractmethod
    def delete_saved_scale(self, scale_name):
        raise NotImplementedError

    @abc.abstractmethod
    def delete_saved_task(self, task_name):
        raise NotImplementedError

    @abc.abstractmethod
    def device_supports_cal(self, device_name):
        raise NotImplementedError

    @abc.abstractmethod
    def disable_ref_trig(self, task):
        raise NotImplementedError

    @abc.abstractmethod
    def disable_start_trig(self, task):
        raise NotImplementedError

    @abc.abstractmethod
    def disconnect_terms(self, source_terminal, destination_terminal):
        raise NotImplementedError

    @abc.abstractmethod
    def export_signal(self, task, signal_id, output_terminal):
        raise NotImplementedError

    @abc.abstractmethod
    def get_analog_power_up_states_with_output_type(
            self, channel_names, array_size):
        raise NotImplementedError

    @abc.abstractmethod
    def get_auto_configured_cdaq_sync_connections(self):
        raise NotImplementedError

    @abc.abstractmethod
    def get_buffer_attribute_uint32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_cal_info_attribute_bool(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_cal_info_attribute_double(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_cal_info_attribute_string(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_cal_info_attribute_uint32(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_chan_attribute_bool(self, task, channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_chan_attribute_double(self, task, channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_chan_attribute_double_array(self, task, channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_chan_attribute_int32(self, task, channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_chan_attribute_string(self, task, channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_chan_attribute_uint32(self, task, channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_device_attribute_bool(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_device_attribute_double(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_device_attribute_double_array(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_device_attribute_int32(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_device_attribute_int32_array(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_device_attribute_string(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_device_attribute_uint32(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_device_attribute_uint32_array(self, device_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_digital_logic_family_power_up_state(self, device_name):
        raise NotImplementedError

    @abc.abstractmethod
    def get_digital_power_up_states(self, device_name, channel_name):
        raise NotImplementedError

    @abc.abstractmethod
    def get_digital_pull_up_pull_down_states(self, device_name, channel_name):
        raise NotImplementedError

    @abc.abstractmethod
    def get_disconnected_cdaq_sync_ports(self):
        raise NotImplementedError

    @abc.abstractmethod
    def get_error_string(self, error_code):
        raise NotImplementedError

    @abc.abstractmethod
    def get_exported_signal_attribute_bool(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_exported_signal_attribute_double(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_exported_signal_attribute_int32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_exported_signal_attribute_string(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_exported_signal_attribute_uint32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_ext_cal_last_date_and_time(self, device_name):
        raise NotImplementedError

    @abc.abstractmethod
    def get_persisted_chan_attribute_bool(self, channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_persisted_chan_attribute_string(self, channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_persisted_scale_attribute_bool(self, scale_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_persisted_scale_attribute_string(self, scale_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_persisted_task_attribute_bool(self, task_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_persisted_task_attribute_string(self, task_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_physical_chan_attribute_bool(self, physical_channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_physical_chan_attribute_bytes(self, physical_channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_physical_chan_attribute_double(self, physical_channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_physical_chan_attribute_double_array(
            self, physical_channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_physical_chan_attribute_int32(self, physical_channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_physical_chan_attribute_int32_array(
            self, physical_channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_physical_chan_attribute_string(self, physical_channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_physical_chan_attribute_uint32(self, physical_channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_physical_chan_attribute_uint32_array(
            self, physical_channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_read_attribute_bool(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_read_attribute_double(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_read_attribute_int32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_read_attribute_string(self, task, attribute, size_hint=0):
        raise NotImplementedError

    @abc.abstractmethod
    def get_read_attribute_uint32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_read_attribute_uint64(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_scale_attribute_double(self, scale_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_scale_attribute_double_array(self, scale_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_scale_attribute_int32(self, scale_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_scale_attribute_string(self, scale_name, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_self_cal_last_date_and_time(self, device_name):
        raise NotImplementedError

    @abc.abstractmethod
    def get_system_info_attribute_string(self, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_system_info_attribute_uint32(self, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_task_attribute_bool(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_task_attribute_string(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_task_attribute_uint32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_bool(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_double(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_ex_bool(self, task, device_names, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_ex_double(self, task, device_names, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_ex_int32(self, task, device_names, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_ex_string(self, task, device_names, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_ex_uint32(self, task, device_names, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_ex_uint64(self, task, device_names, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_int32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_string(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_uint32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_timing_attribute_uint64(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_trig_attribute_bool(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_trig_attribute_double(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_trig_attribute_double_array(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_trig_attribute_int32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_trig_attribute_int32_array(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_trig_attribute_string(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_trig_attribute_timestamp(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_trig_attribute_uint32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_watchdog_attribute_bool(self, task, lines, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_watchdog_attribute_double(self, task, lines, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_watchdog_attribute_int32(self, task, lines, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_watchdog_attribute_string(self, task, lines, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_write_attribute_bool(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_write_attribute_double(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_write_attribute_int32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_write_attribute_string(self, task, attribute, size_hint=0):
        raise NotImplementedError

    @abc.abstractmethod
    def get_write_attribute_uint32(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def get_write_attribute_uint64(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def is_task_done(self, task):
        raise NotImplementedError

    @abc.abstractmethod
    def load_task(self, session_name):
        raise NotImplementedError

    @abc.abstractmethod
    def perform_bridge_offset_nulling_cal_ex(
            self, task, channel, skip_unsupported_channels):
        raise NotImplementedError

    @abc.abstractmethod
    def perform_bridge_shunt_cal_ex(
            self, task, channel, shunt_resistor_value,
            shunt_resistor_location, shunt_resistor_select,
            shunt_resistor_source, bridge_resistance,
            skip_unsupported_channels):
        raise NotImplementedError

    @abc.abstractmethod
    def perform_strain_shunt_cal_ex(
            self, task, channel, shunt_resistor_value,
            shunt_resistor_location, shunt_resistor_select,
            shunt_resistor_source, skip_unsupported_channels):
        raise NotImplementedError

    @abc.abstractmethod
    def perform_thrmcpl_lead_offset_nulling_cal(
            self, task, channel, skip_unsupported_channels):
        raise NotImplementedError

    @abc.abstractmethod
    def read_analog_f64(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_analog_scalar_f64(self, task, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def read_binary_i16(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_binary_i32(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_binary_u16(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_binary_u32(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_counter_f64(self, task, num_samps_per_chan, timeout, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_counter_f64_ex(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_counter_scalar_f64(self, task, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def read_counter_scalar_u32(self, task, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def read_counter_u32(self, task, num_samps_per_chan, timeout, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_counter_u32_ex(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_ctr_freq(
            self, task, num_samps_per_chan, timeout, interleaved,
            read_array_frequency, read_array_duty_cycle):
        raise NotImplementedError

    @abc.abstractmethod
    def read_ctr_freq_scalar(self, task, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def read_ctr_ticks(
            self, task, num_samps_per_chan, timeout, interleaved,
            read_array_high_ticks, read_array_low_ticks):
        raise NotImplementedError

    @abc.abstractmethod
    def read_ctr_ticks_scalar(self, task, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def read_ctr_time(
            self, task, num_samps_per_chan, timeout, interleaved,
            read_array_high_time, read_array_low_time):
        raise NotImplementedError

    @abc.abstractmethod
    def read_ctr_time_scalar(self, task, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def read_digital_lines(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_digital_scalar_u32(self, task, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def read_digital_u16(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_digital_u32(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_digital_u8(
            self, task, num_samps_per_chan, timeout, fill_mode, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def read_power_binary_i16(
            self, task, num_samps_per_chan, timeout, fill_mode,
            read_array_voltage, read_array_current):
        raise NotImplementedError

    @abc.abstractmethod
    def read_power_f64(
            self, task, num_samps_per_chan, timeout, fill_mode,
            read_array_voltage, read_array_current):
        raise NotImplementedError

    @abc.abstractmethod
    def read_power_scalar_f64(self, task, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def read_raw(self, task, num_samps_per_chan, timeout, read_array):
        raise NotImplementedError

    @abc.abstractmethod
    def register_done_event(
            self, task, options, callback_function, callback_data):
        raise NotImplementedError

    @abc.abstractmethod
    def register_every_n_samples_event(
            self, task, every_n_samples_event_type, n_samples, options,
            callback_function, callback_data):
        raise NotImplementedError

    @abc.abstractmethod
    def register_signal_event(
            self, task, signal_id, options, callback_function, callback_data):
        raise NotImplementedError

    @abc.abstractmethod
    def remove_cdaq_sync_connection(self, port_list):
        raise NotImplementedError

    @abc.abstractmethod
    def reserve_network_device(self, device_name, override_reservation):
        raise NotImplementedError

    @abc.abstractmethod
    def reset_buffer_attribute(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def reset_chan_attribute(self, task, channel, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def reset_device(self, device_name):
        raise NotImplementedError

    @abc.abstractmethod
    def reset_exported_signal_attribute(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def reset_read_attribute(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def reset_timing_attribute(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def reset_timing_attribute_ex(self, task, device_names, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def reset_trig_attribute(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def reset_watchdog_attribute(self, task, lines, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def reset_write_attribute(self, task, attribute):
        raise NotImplementedError

    @abc.abstractmethod
    def restore_last_ext_cal_const(self, device_name):
        raise NotImplementedError

    @abc.abstractmethod
    def save_global_chan(self, task, channel_name, save_as, author, options):
        raise NotImplementedError

    @abc.abstractmethod
    def save_scale(self, scale_name, save_as, author, options):
        raise NotImplementedError

    @abc.abstractmethod
    def save_task(self, task, save_as, author, options):
        raise NotImplementedError

    @abc.abstractmethod
    def self_cal(self, device_name):
        raise NotImplementedError

    @abc.abstractmethod
    def self_test_device(self, device_name):
        raise NotImplementedError

    @abc.abstractmethod
    def set_analog_power_up_states(
            self, device_name, channel_names, state, channel_type):
        raise NotImplementedError

    @abc.abstractmethod
    def set_analog_power_up_states_with_output_type(
            self, channel_names, state_array, channel_type_array):
        raise NotImplementedError

    @abc.abstractmethod
    def set_buffer_attribute_uint32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_cal_info_attribute_bool(self, device_name, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_cal_info_attribute_double(self, device_name, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_cal_info_attribute_string(self, device_name, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_cal_info_attribute_uint32(self, device_name, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_chan_attribute_bool(self, task, channel, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_chan_attribute_double(self, task, channel, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_chan_attribute_double_array(self, task, channel, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_chan_attribute_int32(self, task, channel, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_chan_attribute_string(self, task, channel, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_chan_attribute_uint32(self, task, channel, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_digital_logic_family_power_up_state(
            self, device_name, logic_family):
        raise NotImplementedError

    @abc.abstractmethod
    def set_digital_power_up_states(self, device_name, channel_names, state):
        raise NotImplementedError

    @abc.abstractmethod
    def set_digital_pull_up_pull_down_states(
            self, device_name, channel_names, state):
        raise NotImplementedError

    @abc.abstractmethod
    def set_exported_signal_attribute_bool(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_exported_signal_attribute_double(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_exported_signal_attribute_int32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_exported_signal_attribute_string(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_exported_signal_attribute_uint32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_read_attribute_bool(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_read_attribute_double(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_read_attribute_int32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_read_attribute_string(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_read_attribute_uint32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_read_attribute_uint64(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_runtime_environment(
            self, environment, environment_version, reserved_1, reserved_2):
        raise NotImplementedError

    @abc.abstractmethod
    def set_scale_attribute_double(self, scale_name, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_scale_attribute_double_array(self, scale_name, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_scale_attribute_int32(self, scale_name, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_scale_attribute_string(self, scale_name, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_bool(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_double(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_ex_bool(
            self, task, device_names, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_ex_double(
            self, task, device_names, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_ex_int32(
            self, task, device_names, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_ex_string(
            self, task, device_names, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_ex_uint32(
            self, task, device_names, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_ex_uint64(
            self, task, device_names, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_int32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_string(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_uint32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_timing_attribute_uint64(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_trig_attribute_bool(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_trig_attribute_double(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_trig_attribute_double_array(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_trig_attribute_int32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_trig_attribute_int32_array(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_trig_attribute_string(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_trig_attribute_timestamp(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_trig_attribute_uint32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_watchdog_attribute_bool(self, task, lines, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_watchdog_attribute_double(self, task, lines, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_watchdog_attribute_int32(self, task, lines, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_watchdog_attribute_string(self, task, lines, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_write_attribute_bool(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_write_attribute_double(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_write_attribute_int32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_write_attribute_string(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_write_attribute_uint32(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def set_write_attribute_uint64(self, task, attribute, value):
        raise NotImplementedError

    @abc.abstractmethod
    def start_new_file(self, task, file_path):
        raise NotImplementedError

    @abc.abstractmethod
    def start_task(self, task):
        raise NotImplementedError

    @abc.abstractmethod
    def stop_task(self, task):
        raise NotImplementedError

    @abc.abstractmethod
    def task_control(self, task, action):
        raise NotImplementedError

    @abc.abstractmethod
    def tristate_output_term(self, output_terminal):
        raise NotImplementedError

    @abc.abstractmethod
    def unregister_done_event(self, task):
        raise NotImplementedError

    @abc.abstractmethod
    def unregister_every_n_samples_event(
            self, task, every_n_samples_event_type):
        raise NotImplementedError

    @abc.abstractmethod
    def unregister_signal_event(self, task, signal_id):
        raise NotImplementedError

    @abc.abstractmethod
    def unreserve_network_device(self, device_name):
        raise NotImplementedError

    @abc.abstractmethod
    def wait_for_valid_timestamp(self, task, timestamp_event, timeout):
        raise NotImplementedError

    @abc.abstractmethod
    def wait_until_task_done(self, task, time_to_wait):
        raise NotImplementedError

    @abc.abstractmethod
    def write_analog_f64(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            write_array):
        raise NotImplementedError

    @abc.abstractmethod
    def write_analog_scalar_f64(self, task, auto_start, timeout, value):
        raise NotImplementedError

    @abc.abstractmethod
    def write_binary_i16(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            write_array):
        raise NotImplementedError

    @abc.abstractmethod
    def write_binary_i32(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            write_array):
        raise NotImplementedError

    @abc.abstractmethod
    def write_binary_u16(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            write_array):
        raise NotImplementedError

    @abc.abstractmethod
    def write_binary_u32(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            write_array):
        raise NotImplementedError

    @abc.abstractmethod
    def write_ctr_freq(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            frequency, duty_cycle):
        raise NotImplementedError

    @abc.abstractmethod
    def write_ctr_freq_scalar(
            self, task, auto_start, timeout, frequency, duty_cycle):
        raise NotImplementedError

    @abc.abstractmethod
    def write_ctr_ticks(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            high_ticks, low_ticks):
        raise NotImplementedError

    @abc.abstractmethod
    def write_ctr_ticks_scalar(
            self, task, auto_start, timeout, high_ticks, low_ticks):
        raise NotImplementedError

    @abc.abstractmethod
    def write_ctr_time(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            high_time, low_time):
        raise NotImplementedError

    @abc.abstractmethod
    def write_ctr_time_scalar(
            self, task, auto_start, timeout, high_time, low_time):
        raise NotImplementedError

    @abc.abstractmethod
    def write_digital_lines(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            write_array):
        raise NotImplementedError

    @abc.abstractmethod
    def write_digital_scalar_u32(self, task, auto_start, timeout, value):
        raise NotImplementedError

    @abc.abstractmethod
    def write_digital_u16(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            write_array):
        raise NotImplementedError

    @abc.abstractmethod
    def write_digital_u32(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            write_array):
        raise NotImplementedError

    @abc.abstractmethod
    def write_digital_u8(
            self, task, num_samps_per_chan, auto_start, timeout, data_layout,
            write_array):
        raise NotImplementedError

    @abc.abstractmethod
    def write_raw(self, task, num_samps, auto_start, timeout, write_array):
        raise NotImplementedError

    @abc.abstractmethod
    def write_to_teds_from_array(
            self, physical_channel, bit_stream, basic_teds_options):
        raise NotImplementedError

    @abc.abstractmethod
    def write_to_teds_from_file(
            self, physical_channel, file_path, basic_teds_options):
        raise NotImplementedError

    @abc.abstractmethod
    def hash_task_handle(self, task_handle):
        raise NotImplementedError


"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import abc
import collections.abc
import grpc
import grpc.aio
from nidaqmx._stubs import session_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore[misc, type-arg]
    ...

class SessionUtilitiesStub:
    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    EnumerateDevices: grpc.UnaryUnaryMultiCallable[
        session_pb2.EnumerateDevicesRequest,
        session_pb2.EnumerateDevicesResponse,
    ]
    """Provides a list of devices or chassis connected to server under localhost"""

    Reserve: grpc.UnaryUnaryMultiCallable[
        session_pb2.ReserveRequest,
        session_pb2.ReserveResponse,
    ]
    """Reserve a set of client defined resources for exclusive use"""

    IsReservedByClient: grpc.UnaryUnaryMultiCallable[
        session_pb2.IsReservedByClientRequest,
        session_pb2.IsReservedByClientResponse,
    ]
    """Determines if a set of client defined resources is currently reserved by a
    specific client
    """

    Unreserve: grpc.UnaryUnaryMultiCallable[
        session_pb2.UnreserveRequest,
        session_pb2.UnreserveResponse,
    ]
    """Unreserves a previously reserved resource"""

    ResetServer: grpc.UnaryUnaryMultiCallable[
        session_pb2.ResetServerRequest,
        session_pb2.ResetServerResponse,
    ]
    """Resets the server to a default state with no open sessions"""

class SessionUtilitiesAsyncStub:
    EnumerateDevices: grpc.aio.UnaryUnaryMultiCallable[
        session_pb2.EnumerateDevicesRequest,
        session_pb2.EnumerateDevicesResponse,
    ]
    """Provides a list of devices or chassis connected to server under localhost"""

    Reserve: grpc.aio.UnaryUnaryMultiCallable[
        session_pb2.ReserveRequest,
        session_pb2.ReserveResponse,
    ]
    """Reserve a set of client defined resources for exclusive use"""

    IsReservedByClient: grpc.aio.UnaryUnaryMultiCallable[
        session_pb2.IsReservedByClientRequest,
        session_pb2.IsReservedByClientResponse,
    ]
    """Determines if a set of client defined resources is currently reserved by a
    specific client
    """

    Unreserve: grpc.aio.UnaryUnaryMultiCallable[
        session_pb2.UnreserveRequest,
        session_pb2.UnreserveResponse,
    ]
    """Unreserves a previously reserved resource"""

    ResetServer: grpc.aio.UnaryUnaryMultiCallable[
        session_pb2.ResetServerRequest,
        session_pb2.ResetServerResponse,
    ]
    """Resets the server to a default state with no open sessions"""

class SessionUtilitiesServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def EnumerateDevices(
        self,
        request: session_pb2.EnumerateDevicesRequest,
        context: _ServicerContext,
    ) -> typing.Union[session_pb2.EnumerateDevicesResponse, collections.abc.Awaitable[session_pb2.EnumerateDevicesResponse]]:
        """Provides a list of devices or chassis connected to server under localhost"""

    @abc.abstractmethod
    def Reserve(
        self,
        request: session_pb2.ReserveRequest,
        context: _ServicerContext,
    ) -> typing.Union[session_pb2.ReserveResponse, collections.abc.Awaitable[session_pb2.ReserveResponse]]:
        """Reserve a set of client defined resources for exclusive use"""

    @abc.abstractmethod
    def IsReservedByClient(
        self,
        request: session_pb2.IsReservedByClientRequest,
        context: _ServicerContext,
    ) -> typing.Union[session_pb2.IsReservedByClientResponse, collections.abc.Awaitable[session_pb2.IsReservedByClientResponse]]:
        """Determines if a set of client defined resources is currently reserved by a
        specific client
        """

    @abc.abstractmethod
    def Unreserve(
        self,
        request: session_pb2.UnreserveRequest,
        context: _ServicerContext,
    ) -> typing.Union[session_pb2.UnreserveResponse, collections.abc.Awaitable[session_pb2.UnreserveResponse]]:
        """Unreserves a previously reserved resource"""

    @abc.abstractmethod
    def ResetServer(
        self,
        request: session_pb2.ResetServerRequest,
        context: _ServicerContext,
    ) -> typing.Union[session_pb2.ResetServerResponse, collections.abc.Awaitable[session_pb2.ResetServerResponse]]:
        """Resets the server to a default state with no open sessions"""

def add_SessionUtilitiesServicer_to_server(servicer: SessionUtilitiesServicer, server: typing.Union[grpc.Server, grpc.aio.Server]) -> None: ...
